#!/usr/bin/env python3
from stepup.core.api import runsh, static
from stepup.reprep.api import compile_typst

static("document.typ")
compile_typst("document.typ")
runsh("echo 'fixed: new' > data.yaml", out="data.yaml")



#!/usr/bin/env python3
from stepup.core.api import glob, script, static
from stepup.reprep.api import convert_pdf_png, convert_svg_pdf

for path_svg in glob("*.svg"):
    convert_svg_pdf(path_svg)
static("tile.py")
script("tile.py")
convert_pdf_png("figure.pdf", resolution=150)



# StepUp RepRep is the StepUp extension for Reproducible Reporting.
# © 2024–2025 Toon Verstraelen
#
# This file is part of StepUp RepRep.
#
# StepUp RepRep is free software;  you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# StepUp RepRep is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
#
# --
"""Unit tests for stepup.reprep.sync_zenodo."""

import pytest

from stepup.reprep.sync_zenodo import Creator


@pytest.mark.parametrize(
    "orcid",
    ["0000-0001-9288-5608", "0000-0001-6785-333X", "0000-0001-6785-333x", "0000-0002-0257-4687"],
)
def test_creator_orcid_valid(orcid):
    Creator("Test User", "StepUp RepRep", {"orcid": orcid})


@pytest.mark.parametrize(
    "orcid",
    [
        "0000-0002-0257-4687  0000-0002-0257-46870000-0001-9288-560x",
        "0000-0001-9288-560X",
        "0000-0001-6785-3337",
        "0000-0002-1825-009",
        "0000-0002-1825-00977",
        "000-0002-1825-0097",
        "0000X0002-1825-0097",
        "https://orcid.org/0000-0002-1825-0097",
        "ABCD-EFGH-IJKL-MNOP",
        "1234-5678-9012-345y",
        "",
        "    ",
        "0000-0002-1825-0090",
        "0000-0002-1694-233A",
    ],
)
def test_creator_orcid_invalid(orcid):
    with pytest.raises(ValueError):
        Creator("Test User", "StepUp RepRep", {"orcid": orcid})



#!/usr/bin/env python3

from stepup.core.api import mkdir, runsh, static
from stepup.reprep.api import compile_latex

static("paper.tex")
mkdir("subdir/")
compile_latex("paper.tex", inventory=True)
runsh(
    "echo 'Verbatim input:\\verbatiminput{code.txt}' > subdir/generated.tex",
    out=["subdir/generated.tex"],
)
runsh("echo '2 + 2' > subdir/code.txt", out=["subdir/code.txt"])



#!/usr/bin/env python3
"""Write an environment variable to a jpg image."""

from PIL import Image, ImageDraw

from stepup.core.api import getenv


def hex_to_rgb(hex_color: str) -> tuple[int, int, int]:
    """
    Converts a hex color string (e.g., '#FFFFFF') to an RGB tuple.

    Parameters
    ----------
    hex_color: The hex color string.

    Returns
    -------
    A tuple of three integers representing the RGB values.
    """
    hex_color = hex_color.lstrip("#")  # Remove the '#' if present
    return tuple(int(hex_color[i : i + 2], 16) for i in (0, 2, 4))


# Take colors from environment variables.
fg_color = hex_to_rgb(getenv("FG_COLOR", "#0044AA"))
bg_color = hex_to_rgb(getenv("BG_COLOR", "#FFCC00"))

# Draw a plus.
img = Image.new("RGB", (200, 200), bg_color)
draw = ImageDraw.Draw(img)
draw.line([(20, 20), (180, 180)], fill=fg_color, width=10)
draw.line([(180, 20), (20, 180)], fill=fg_color, width=10)
img.save("image.jpg")



#!/usr/bin/env python3
from stepup.core.api import copy, static
from stepup.reprep.api import sanitize_bibtex

static("references.bib", "paper.aux", "bibsane.yaml")
sanitize_bibtex(
    "references.bib", path_aux="paper.aux", path_cfg="bibsane.yaml", path_out="cleaned.bib"
)
copy("cleaned.bib", "copy.bib")



#!/usr/bin/env python3
from stepup.core.script import driver
from stepup.reprep.tile_pdf import Figure, Panel

FIGURE = Figure(
    "figure.pdf",
    [
        Panel(0, 0, "(a) triangle", "triangle.pdf"),
        Panel(0, 1, "(b) square", "square.pdf"),
        Panel(1, 0, "(c) pentagon", "pentagon.pdf"),
        Panel(1, 1, "(d) hexagon", "hexagon.pdf"),
    ],
)

if __name__ == "__main__":
    driver(FIGURE)



#!/usr/bin/env python3
from stepup.core.api import static
from stepup.reprep.api import unplot

static("plot1.svg", "plot2.svg")
unplot("plot1.svg", "current_plot1.json")
unplot("plot2.svg", "current_plot2.json")



# StepUp RepRep is the StepUp extension for Reproducible Reporting.
# © 2024–2025 Toon Verstraelen
#
# This file is part of StepUp RepRep.
#
# StepUp RepRep is free software;  you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# StepUp RepRep is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
#
# --
"""Unit tests for stepup.reprep.latex_flat"""

from stepup.reprep.flatten_latex import flatten_latex

MAIN_TEX = r"""
\begin{document}
\input{table}
\includegraphics{smile}\cite{knuth:1984}
\import{sub}{foo.tex}
\end{document}
"""

TABLE_TEX = "This is a table.\n"

FOO_TEX = r"""This is a foo.
\includegraphics[width=5cm]{smile.pdf}
\thebibliography{references}
\input{other.tex}
"""

OTHER_TEX = "Another line.\n"

EXPECTED = r"""
\begin{document}
This is a table.
\includegraphics{../smile}\cite{knuth:1984}
This is a foo.
\includegraphics[width=5cm]{../sub/smile.pdf}
\thebibliography{../sub/references}
Another line.
\end{document}
"""

CREATE_FILES = {
    "main.tex": MAIN_TEX,
    "table.tex": TABLE_TEX,
    "sub/foo.tex": FOO_TEX,
    "sub/other.tex": OTHER_TEX,
}


def test_latex_flat(path_tmp):
    for filename, contents in CREATE_FILES.items():
        path_dst = path_tmp / filename
        path_dst.parent.makedirs_p()
        with open(path_dst, "w") as fh:
            fh.write(contents)
    flatdir = path_tmp / "flat"
    flatdir.makedirs_p()
    with open(flatdir / "main.tex", "w") as fh:
        flatten_latex(path_tmp / "main.tex", fh, flatdir)
    with open(flatdir / "main.tex") as fh:
        result = fh.read()
    assert result.strip() == EXPECTED.strip()


def test_latex_flat_simple(path_tmp):
    with open(path_tmp / "main.tex", "w") as fh:
        fh.write("Not so much to see here.")
    with open(path_tmp / "flat.tex", "w") as fh:
        flatten_latex(path_tmp / "main.tex", fh, path_tmp)
    with open(path_tmp / "flat.tex") as fh:
        result = fh.read()
    assert result.strip() == "Not so much to see here."


def test_latex_flat_empty(path_tmp):
    with open(path_tmp / "main.tex", "w") as fh:
        pass
    with open(path_tmp / "flat.tex", "w") as fh:
        flatten_latex(path_tmp / "main.tex", fh, path_tmp)
    with open(path_tmp / "flat.tex") as fh:
        result = fh.read()
    assert result.strip() == ""



# StepUp RepRep is the StepUp extension for Reproducible Reporting.
# © 2024–2025 Toon Verstraelen
#
# This file is part of StepUp RepRep.
#
# StepUp RepRep is free software;  you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# StepUp RepRep is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
#
# --
"""Unit tests for stepup.reprep.zip_inventory."""

import contextlib
import zipfile

from stepup.reprep.make_inventory import write_inventory
from stepup.reprep.zip_inventory import zip_inventory


def test_simple_chdir(path_tmp):
    with contextlib.chdir(path_tmp):
        with open("a.txt", "w") as fh:
            fh.write("Aaa")
        with open("b.txt", "w") as fh:
            fh.write("Bbb")
        write_inventory("inventory.txt", ["a.txt", "b.txt"])
        zip_inventory("inventory.txt", "test.zip")
        contents = {}
        with zipfile.ZipFile("test.zip", "r") as fz:
            for name in fz.namelist():
                contents[name] = fz.read(name).decode("utf-8")
        assert contents["a.txt"] == "Aaa"
        assert contents["b.txt"] == "Bbb"
        assert "inventory.txt" in contents


def test_simple(path_tmp):
    with open(path_tmp / "a.txt", "w") as fh:
        fh.write("Aaa")
    with open(path_tmp / "b.txt", "w") as fh:
        fh.write("Bbb")
    write_inventory(path_tmp / "inventory.txt", [path_tmp / "a.txt", path_tmp / "b.txt"])
    zip_inventory(path_tmp / "inventory.txt")
    contents = {}
    with zipfile.ZipFile(path_tmp / "inventory.zip", "r") as fz:
        for name in fz.namelist():
            contents[name] = fz.read(name).decode("utf-8")
    assert contents["a.txt"] == "Aaa"
    assert contents["b.txt"] == "Bbb"
    assert "inventory.txt" in contents



# StepUp RepRep is the StepUp extension for Reproducible Reporting.
# © 2024–2025 Toon Verstraelen
#
# This file is part of StepUp RepRep.
#
# StepUp RepRep is free software;  you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# StepUp RepRep is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
#
# --
"""Run examples."""

import os
import re
import shutil
import subprocess

import pytest
from path import Path

from stepup.reprep.pytest import run_example

OVERWRITE_EXPECTED = "STEPUP_OVERWRITE_EXPECTED" in os.environ


@pytest.mark.parametrize(
    "name",
    [
        "add_notes_pdf",
        "bibsane_same",
        "bibsane_other",
        "cat_pdf",
        "check_hrefs_html",
        "check_hrefs_md",
        "convert_markdown",
        pytest.param("convert_markdown_concurrency", marks=pytest.mark.heavy),
        "convert_markdown_env",
        "convert_weasyprint",
        "execute_papermill",
        "flatten_latex",
        "flatten_latex_subdir",
        "make_inventory_list",
        "nup_pdf",
        "raster_pdf",
        "sync_zenodo",
        "unplot",
        "wrap_git",
        "zip_def",
        "zip_inventory",
        "zip_tree",
    ],
)
@pytest.mark.asyncio
async def test_example(path_tmp, name: str):
    await run_example(Path("tests/examples") / name, path_tmp, OVERWRITE_EXPECTED)


def has_texlive_2023():
    if not shutil.which("lualatex"):
        return False
    if not shutil.which("pdflatex"):
        return False
    if not shutil.which("xelatex"):
        return False
    if not shutil.which("bibtex"):
        return False
    if not shutil.which("latexdiff"):
        return False
    cp = subprocess.run(
        ["latex", "--version"],
        stdout=subprocess.PIPE,
        stdin=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
        check=True,
        text=True,
    )
    match = re.search(r"TeX Live (?P<year>\d\d\d\d)", cp.stdout)
    if match is None:
        return False
    return match.group("year") == "2023"


@pytest.mark.skipif(not has_texlive_2023(), reason="No TeX Live 2023")
@pytest.mark.parametrize(
    "name",
    [
        "check_hrefs_pdf",
        "diff_latex",
        "compile_lualatex_simple",
        "compile_pdflatex_bbl",
        "compile_pdflatex_bibtex",
        "compile_pdflatex_input",
        "compile_xelatex_input",
    ],
)
@pytest.mark.asyncio
async def test_latex_example(path_tmp: Path, name: str):
    await run_example(Path("tests/examples") / name, path_tmp, OVERWRITE_EXPECTED)


@pytest.mark.skipif(not shutil.which("inkscape"), reason="No Inkscape")
@pytest.mark.parametrize(
    "name",
    [
        "convert_inkscape",
        pytest.param("convert_inkscape_concurrency", marks=pytest.mark.heavy),
        "tile_pdf",
    ],
)
@pytest.mark.asyncio
async def test_inkscape_example(path_tmp: Path, name: str):
    await run_example(Path("tests/examples") / name, path_tmp, OVERWRITE_EXPECTED)


@pytest.mark.skipif(not shutil.which("mutool"), reason="No Mutool")
@pytest.mark.parametrize(
    "name",
    ["convert_mutool"],
)
@pytest.mark.asyncio
async def test_mutool_example(path_tmp: Path, name: str):
    await run_example(Path("tests/examples") / name, path_tmp, OVERWRITE_EXPECTED)


@pytest.mark.skipif(not shutil.which("libreoffice"), reason="No LibreOffice")
@pytest.mark.parametrize(
    "name",
    [
        "convert_libreoffice",
        pytest.param("convert_libreoffice_concurrency", marks=pytest.mark.heavy),
    ],
)
@pytest.mark.asyncio
async def test_libreoffice_example(path_tmp: Path, name: str):
    await run_example(Path("tests/examples") / name, path_tmp, OVERWRITE_EXPECTED)


@pytest.mark.skipif(not shutil.which("typst"), reason="No Typst")
@pytest.mark.parametrize(
    "name",
    [
        "compile_typst_args",
        "compile_typst_dep",
        pytest.param("compile_typst_dep_error", marks=pytest.mark.xfail),
        "compile_typst_error",
        "compile_typst_external",
        "compile_typst_png",
        "compile_typst_png_multi",
        "compile_typst_relpath",
        "compile_typst_simple",
        "compile_typst_svg",
        "compile_typst_svg_deps",
        "compile_typst_svg_multi",
        "compile_typst_sysinp",
        "compile_typst_sysinp_json",
    ],
)
@pytest.mark.asyncio
async def test_typst_example(path_tmp: Path, name: str):
    await run_example(Path("tests/examples") / name, path_tmp, OVERWRITE_EXPECTED)


@pytest.mark.skipif(not shutil.which("jupyter"), reason="No Jupyter")
@pytest.mark.parametrize(
    "name",
    [
        "convert_jupyter",
    ],
)
@pytest.mark.asyncio
async def test_jupyter_example(path_tmp: Path, name: str):
    await run_example(Path("tests/examples") / name, path_tmp, OVERWRITE_EXPECTED)



#!/usr/bin/env python3
from stepup.core.api import glob, static
from stepup.reprep.api import convert_markdown

static("source/", "source/demo.md")
glob("common/**", _defer=True)
convert_markdown("source/demo.md")



#!/usr/bin/env python3
from stepup.core.api import copy, static
from stepup.reprep.api import convert_odf_pdf

static("something.odt")
for i in range(20):
    copy("something.odt", f"copy_{i:02d}.odt")
    convert_odf_pdf(f"copy_{i:02d}.odt")



#!/usr/bin/env python3
from stepup.core.api import static
from stepup.reprep.api import flatten_latex

static("article_structured.tex", "part1.tex", "part2.tex", "part3.txt")
flatten_latex("article_structured.tex", "article.tex")



#!/usr/bin/env python3
from stepup.core.api import glob, static
from stepup.reprep.api import make_inventory, zip_inventory

static("inventory.def")

# Make all file in `data` static when they are needed.
glob("data/**", _defer=True)

# Create an inventory file with all files defined in `inventory.def`.
make_inventory("inventory.txt", path_def="inventory.def")
zip_inventory("inventory.txt", "upload.zip")



#!/usr/bin/env python3
from stepup.core.api import static
from stepup.reprep.api import convert_jupyter

static("demo.ipynb", "data.json", "points.txt")
convert_jupyter("demo.ipynb", inp="points.txt", out="plot.png", nbargs={"dpi": 50})
